@@ -77,7 +77,7 @@ def main():
 
         # 1. Load Model
         config = MultimodalConfig(
-            vocab_size=1000,      # Small vocab for demo
+            vocab_size=50257,     # Match GPT-2 tokenizer vocab size
             vision_encoder="vit", # Use ViT but smaller
             image_size=64,        # Very small images for demo
             fusion_type="cross_attention",
