@@ -20,8 +20,8 @@ print(f"Python version: {sys.version}")
 print(f"Platform: {sys.platform}")
 
 # Add project root to sys.path to allow importing parallel_llm from source
-sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
-print(f"Added to path: {os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))}")
+# sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
+# print(f"Added to path: {os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))}")
 
 # Try to import dependencies
 PYTORCH_AVAILABLE = False
@@ -147,12 +147,14 @@ def main():
 
         # Mock dataset for demonstration if real one isn't available
         class MockMultimodalDataset(torch.utils.data.Dataset):
-            def __init__(self, length=100):
+            def __init__(self, length=25):  # Reduced for demo
                 self.length = length
             def __len__(self): return self.length
             def __getitem__(self, idx):
+                # Create mock image tensor normalized to [0,1] range
+                mock_image = torch.rand(3, 224, 224)  # Uniform [0,1] instead of normal distribution
                 return {
-                    "image": torch.randn(3, 224, 224), # Mock image tensor
+                    "image": mock_image,  # Mock image tensor in [0,1] range
                     "text": "A description of an image."
                 }
 
