@@ -209,10 +209,13 @@ class DiffusionTransformer(nn.Module):
     def __init__(self, config):
         super().__init__()
         self.config = config
-        self.padding_idx = config.vocab_size
+        # Use 0 as padding index to avoid out-of-bounds errors
+        # This is a special MASK token used during diffusion
+        self.padding_idx = 0
+        self.mask_token_id = 0  # Alias for clarity
 
-        # Embeddings
-        self.embed_tokens = nn.Embedding(config.vocab_size + 1, config.hidden_size, padding_idx=self.padding_idx)
+        # Embeddings - vocab_size tokens (no +1 needed)
+        self.embed_tokens = nn.Embedding(config.vocab_size, config.hidden_size, padding_idx=None)
 
         # Time embedding for diffusion steps
         self.time_embed = nn.Sequential(
@@ -341,10 +344,10 @@ class DiffusionTransformer(nn.Module):
         Returns:
             generated_ids: [batch, seq_len]
         """
-        # Start with all MASK tokens
+        # Start with all MASK tokens (using token 0 as mask)
         generated_ids = torch.full(
             (batch_size, seq_len),
-            self.padding_idx,
+            self.mask_token_id,
             dtype=torch.long,
             device=device
         )
@@ -370,6 +373,9 @@ class DiffusionTransformer(nn.Module):
                 probs.view(-1, self.config.vocab_size),
                 num_samples=1
             ).view(batch_size, seq_len)
+            
+            # Clamp sampled IDs to valid range [0, vocab_size-1]
+            sampled_ids = torch.clamp(sampled_ids, 0, self.config.vocab_size - 1)
 
             # Determine which positions to keep based on confidence
             keep_mask = (confidence > confidence_threshold) & is_masked
@@ -387,6 +393,8 @@ class DiffusionTransformer(nn.Module):
             timestep = torch.zeros((batch_size,), device=device)
             logits, _ = self.forward(generated_ids, timestep)
             final_ids = torch.argmax(logits, dim=-1)
+            # Clamp to valid range
+            final_ids = torch.clamp(final_ids, 0, self.config.vocab_size - 1)
             generated_ids = torch.where(is_masked, final_ids, generated_ids)
 
         return generated_ids
