@@ -21,6 +21,32 @@ class GenerationConfig:
     confidence_threshold: float = 0.9
     use_adaptive_steps: bool = True
     batch_size: int = 1
+    num_parallel_tokens: int = 64  # For compatibility with config from inference module
+    use_torch_compile: bool = False # Use torch.compile for faster inference (requires PyTorch 2.0+)
+    
+    def __post_init__(self):
+        """Validate configuration values"""
+        # Ensure temperature is positive
+        if self.temperature <= 0:
+            self.temperature = 1.0
+        
+        # Ensure top_k is non-negative
+        if self.top_k < 0:
+            self.top_k = 0
+        
+        # Clamp top_p to [0, 1]
+        self.top_p = max(0.0, min(1.0, self.top_p))
+        
+        # Ensure repetition_penalty is positive
+        if self.repetition_penalty <= 0:
+            self.repetition_penalty = 1.0
+        
+        # Ensure num_refinement_steps is at least 1
+        if self.num_refinement_steps < 1:
+            self.num_refinement_steps = 1
+        
+        # Clamp confidence_threshold to [0, 1]
+        self.confidence_threshold = max(0.0, min(1.0, self.confidence_threshold))
 
 
 class PagedKVCache:
@@ -135,6 +161,9 @@ class ParallelGenerator:
             )
 
         # Compile model for faster inference
+        if config.use_torch_compile and hasattr(torch, "compile"):
+            self.model = torch.compile(self.model)
+
         if use_cuda_graphs:
             self._setup_cuda_graphs()
 
@@ -142,7 +171,9 @@ class ParallelGenerator:
         """Setup CUDA graphs for maximum throughput"""
         # Warmup to trigger compilation
         device = next(self.model.parameters()).device
-        dummy_input = torch.randint(0, 100, (1, 64), device=device)
+        vocab_size = self.model.config.vocab_size
+        # Generate dummy input with valid token range [0, vocab_size-1]
+        dummy_input = torch.randint(0, min(100, vocab_size), (1, 64), device=device)
         dummy_timestep = torch.zeros(1, device=device)
 
         for _ in range(3):
@@ -183,13 +214,21 @@ class ParallelGenerator:
         max_new_tokens = max_new_tokens or self.config.max_new_tokens
         device = prompt_tokens.device
 
-        # Initialize full sequence with MASK tokens
+        # Validate prompt tokens against vocab size
+        if prompt_tokens.max() >= self.model.config.vocab_size:
+            raise ValueError(
+                f"Prompt contains token IDs greater than model vocabulary size "
+                f"({self.model.config.vocab_size}). This will cause CUDA errors."
+            )
+
+        # Initialize full sequence with MASK tokens (using 0 as mask token)
         total_len = prompt_len + max_new_tokens
+        mask_token_id = getattr(self.model, 'mask_token_id', 0)
         generated = torch.cat([
             prompt_tokens,
             torch.full(
                 (batch_size, max_new_tokens),
-                self.model.padding_idx,
+                mask_token_id,
                 dtype=torch.long,
                 device=device
             )
@@ -233,7 +272,11 @@ class ParallelGenerator:
                     confidence = None
 
             # Apply temperature
-            logits = logits / self.config.temperature
+            if self.config.temperature > 0:
+                logits = logits / self.config.temperature
+            
+            # Replace NaN and inf values with very negative numbers
+            logits = torch.nan_to_num(logits, nan=-1e9, posinf=1e9, neginf=-1e9)
 
             # Apply top-k filtering
             if self.config.top_k > 0:
@@ -247,12 +290,23 @@ class ParallelGenerator:
             if self.config.repetition_penalty != 1.0:
                 logits = self._apply_repetition_penalty(logits, generated, self.config.repetition_penalty)
 
-            # Sample tokens
+            # Sample tokens with safety checks
             probs = F.softmax(logits, dim=-1)
+            
+            # Replace any NaN or inf in probs
+            probs = torch.nan_to_num(probs, nan=0.0, posinf=1.0, neginf=0.0)
+            
+            # Ensure probs sum to 1 (renormalize if needed)
+            probs_sum = probs.sum(dim=-1, keepdim=True)
+            probs = probs / (probs_sum + 1e-10)
+            
             next_tokens = torch.multinomial(
                 probs.view(-1, self.model.config.vocab_size),
                 num_samples=1
             ).view(batch_size, total_len)
+            
+            # Clamp sampled tokens to valid range [0, vocab_size-1]
+            next_tokens = torch.clamp(next_tokens, 0, self.model.config.vocab_size - 1)
 
             # Compute confidence if not returned by model
             if confidence is None:
@@ -274,6 +328,8 @@ class ParallelGenerator:
             timestep = torch.zeros(batch_size, device=device, dtype=torch.long)
             logits, _ = self.model(generated, timestep)
             final_tokens = torch.argmax(logits, dim=-1)
+            # Clamp to valid range
+            final_tokens = torch.clamp(final_tokens, 0, self.model.config.vocab_size - 1)
             generated = torch.where(is_masked, final_tokens, generated)
 
         return generated
@@ -292,19 +348,39 @@ class ParallelGenerator:
 
     def _top_k_filtering(self, logits: torch.Tensor, top_k: int) -> torch.Tensor:
         """Filter logits to keep only top-k tokens"""
-        indices_to_remove = logits < torch.topk(logits, top_k)[0][..., -1, None]
+        # Get the vocabulary size from the last dimension
+        vocab_size = logits.shape[-1]
+        
+        # Clamp top_k to be at most vocab_size
+        top_k = min(top_k, vocab_size)
+        
+        # If top_k is 0 or negative, return logits unchanged
+        if top_k <= 0:
+            return logits
+        
+        # Get top-k values and filter
+        top_k_values = torch.topk(logits, top_k, dim=-1)[0]
+        indices_to_remove = logits < top_k_values[..., -1, None]
         logits = logits.masked_fill(indices_to_remove, float('-inf'))
         return logits
 
     def _top_p_filtering(self, logits: torch.Tensor, top_p: float) -> torch.Tensor:
         """Filter logits using nucleus (top-p) sampling"""
+        # If top_p is 1.0 or greater, no filtering needed
+        if top_p >= 1.0:
+            return logits
+        
+        # Sort logits in descending order
         sorted_logits, sorted_indices = torch.sort(logits, descending=True, dim=-1)
+        
+        # Compute cumulative probabilities
         cumulative_probs = torch.cumsum(F.softmax(sorted_logits, dim=-1), dim=-1)
 
         # Remove tokens with cumulative probability above threshold
+        # Keep at least one token (the first one)
         sorted_indices_to_remove = cumulative_probs > top_p
         sorted_indices_to_remove[..., 1:] = sorted_indices_to_remove[..., :-1].clone()
-        sorted_indices_to_remove[..., 0] = 0
+        sorted_indices_to_remove[..., 0] = False  # Always keep the top token
 
         # Scatter back to original indexing
         indices_to_remove = sorted_indices_to_remove.scatter(
@@ -320,9 +396,13 @@ class ParallelGenerator:
         penalty: float,
     ) -> torch.Tensor:
         """Apply repetition penalty to logits"""
+        mask_token_id = getattr(self.model, 'mask_token_id', 0)
+        vocab_size = self.model.config.vocab_size
+        
         for i in range(generated.shape[0]):
             for token in set(generated[i].tolist()):
-                if token != self.model.padding_idx:
+                # Skip mask token and ensure token is in valid range
+                if token != mask_token_id and 0 <= token < vocab_size:
                     # If score < 0 then repetition penalty has to be multiplied
                     # to reduce the score, otherwise it has to be divided
                     if logits[i, :, token] < 0:
@@ -348,10 +428,11 @@ class ParallelGenerator:
         max_prompt_len = max(p.shape[0] for p in prompts)
         batch_size = len(prompts)
         device = prompts[0].device
-
+        
+        mask_token_id = getattr(self.model, 'mask_token_id', 0)
         padded_prompts = torch.full(
             (batch_size, max_prompt_len),
-            self.model.padding_idx,
+            mask_token_id,
             dtype=torch.long,
             device=device,
         )
