@@ -441,10 +441,14 @@ class DistributedTrainer:
         mask_positions[:, 0] = False  # Don't mask BOS
         mask_positions[:, -1] = False  # Don't mask EOS if present
 
-        # Apply masking - replace with padding token (used as MASK)
+        # Apply masking - replace with mask token (0)
         noisy_input_ids = input_ids.clone()
-        mask_token_id = getattr(self.model, 'padding_idx', 0)  # Use model's padding token
+        mask_token_id = getattr(self.model, 'mask_token_id', 0)  # Use model's mask token
         noisy_input_ids[mask_positions] = mask_token_id
+        
+        # Clamp input_ids to valid range [0, vocab_size-1]
+        input_ids = torch.clamp(input_ids, 0, self.model_config.vocab_size - 1)
+        noisy_input_ids = torch.clamp(noisy_input_ids, 0, self.model_config.vocab_size - 1)
 
         # Forward pass
         logits, confidence = self.model(
